from typing import Mapping, Type

from teslacam.contracts import Notifier, Uploader
from teslacam.notifiers.pushover import PushoverNotifier
from teslacam.uploaders.blobstorage import BlobStorageUploader
from teslacam.uploaders.filesystem import FileSystemUploader

UPLOADERS: Mapping[str, Type[Uploader]] = {
    "blobStorage": BlobStorageUploader,
    "fileSystem": FileSystemUploader
}

NOTIFIERS: Mapping[str, Type[Notifier]] = {
    "pushover": PushoverNotifier
}

TESLACAM_DIR = "TeslaCam"
RECENT_DIR = "RecentClips"
SAVED_DIR = "SavedClips"
SENTRY_DIR = "SentryClips"

ONE_MEGABYTE = 1048576

MIN_FILE_SIZE_BYTES = ONE_MEGABYTE # 1 MB


from os import path, listdir
from pathlib import Path
import shutil
from typing import List, Mapping

try:
    from sh import mount, umount
except:
    def mount(path: str):
        pass

    def umount(path: str):
        pass

from teslacam.config import Configuration
from teslacam.consts import TESLACAM_DIR, RECENT_DIR, SAVED_DIR, SENTRY_DIR
from teslacam.enums import ClipType
from teslacam.models import Clip

CLIP_TYPE_DIR_MAPPING: Mapping[ClipType, str] = {
    ClipType.RECENT: RECENT_DIR,
    ClipType.SAVED: SAVED_DIR,
    ClipType.SENTRY: SENTRY_DIR
}

class FileSystem:
    def __init__(self, cfg: Configuration):
        self.__cfg = cfg

    def read_clips(self, type: ClipType) -> List[Clip]:
        clips_dir = path.join(self.__cfg.tesla_cam_directory,
            TESLACAM_DIR, CLIP_TYPE_DIR_MAPPING[type])

        clips_path = Path(clips_dir)

        if not clips_path.exists():
            return []

        clips = FileSystem.__get_items(clips_path, type)

        return clips

    def delete_empty_event_dirs(self, type: ClipType):
        clips_dir = path.join(self.__cfg.tesla_cam_directory,
            TESLACAM_DIR, CLIP_TYPE_DIR_MAPPING[type])

        clips_path = Path(clips_dir)

        if not clips_path.exists():
            return

        for dir in [item for item in clips_path.iterdir() if item.is_dir()]:
            items = [item for item in listdir(dir) if not item.startswith(".")]
            
            if len(items) == 0:
                shutil.rmtree(dir)

    def mount_directory(self):
        try:
            mount(self.__cfg.tesla_cam_directory)
        except:
            return

    def unmount_directory(self):
        try:
            umount(self.__cfg.tesla_cam_directory)
        except:
            return

    @staticmethod
    def __get_items(clips_path: Path, type: ClipType, items: List[Clip]=None, event: str=None) -> List[Clip]:
        items = [] if items == None else items

        for item in clips_path.iterdir():
            if item.is_file() and path.splitext(item.name)[1] == ".mp4":
                try:
                    clip = Clip(item, type, event)
                    items.append(clip)
                except ValueError:
                    continue

            elif item.is_dir():
                FileSystem.__get_items(item, type, items, item.name)

        return items


from collections import defaultdict
from typing import Callable, Dict, List, TypeVar

TItem = TypeVar("TItem")
TKey = TypeVar("TKey")

def group_by(items: List[TItem], by: Callable[[TItem], TKey]) -> Dict[TKey, List[TItem]]:
    result: Dict[TKey, List[TItem]] = defaultdict(list)

    for item in items:
        result[by(item)].append(item)

    return result


from os import path
import sys

from teslacam import config
from teslacam.log import log
from teslacam.services.filesystem import FileSystem
from teslacam.services.notification import NotificationService
from teslacam.services.upload import UploadService

def get_version():
    with open(path.join(path.dirname(__file__), "VERSION"), encoding="utf-8") as file:
        return file.read()

def print_version():
    print(f"TeslaCam v{get_version()}")

def main():
    if len(sys.argv) > 1 and sys.argv[1] == "-v":
        print_version()
        return

    cfg = config.load_config()
    
    fs = FileSystem(cfg)
    notification = NotificationService(cfg)

    upload = UploadService(cfg, fs, notification)

    # Start upload service
    upload.start()

    log(f"Started TeslaCam v{get_version()}")

if __name__ == "__main__":
    main()


from datetime import datetime
from pathlib import Path
import re
from typing import Optional

from teslacam.enums import ClipType, Camera

DATE_REGEX = re.compile(r"^(\d{4}-\d{2}-\d{2}_\d{2}-\d{2}-\d{2})-")
CAMERA_REGEX = re.compile(r"-(\w+)\.mp4")

DATE_FORMAT = r"%Y-%m-%d_%H-%M-%S"

CAMERA_DICT = {
    "front": Camera.FRONT,
    "left_repeater": Camera.LEFT_REPEATER,
    "right_repeater": Camera.RIGHT_REPEATER,
    "back": Camera.BACK
}

class Clip:
    def __init__(self, path: Path, type: ClipType, event: str=None):
        self.__pathObject = path

        self.__path = str(path)
        self.__name = path.name
        self.__type = type
        self.__size = path.stat().st_size

        self.__event = datetime.strptime(event, DATE_FORMAT) if event is not None else None

        date = DATE_REGEX.findall(self.name)[0]
        self.__date = datetime.strptime(date, DATE_FORMAT)

        camera = CAMERA_REGEX.findall(self.name)[0]
        self.__camera = CAMERA_DICT[camera]

    @property
    def path(self) -> str:
        """
        Path to the clip file on the file system.
        """
        return self.__path

    @property
    def name(self) -> str:
        """
        Name of the clip.
        """
        return self.__name

    @property
    def type(self) -> ClipType:
        """
        Type of the clip.
        """
        return self.__type

    @property
    def date(self) -> datetime:
        """
        Date the clip was created.
        """
        return self.__date

    @property
    def camera(self) -> Camera:
        """
        Camera the clip was recorded with.
        """
        return self.__camera

    @property
    def event(self) -> Optional[datetime]:
        """
        Event (save/sentry) this clip is a part of.
        """
        return self.__event

    @property
    def size(self) -> int:
        """
        Size of the clip in bytes.
        """
        return self.__size

    def delete(self):
        """
        Deletes a clip from disk.
        """
        self.__pathObject.unlink()